#include <QApplication>
#include "onedeviceelectroprogontask.h"
#include "tools/globaltools.h"

OneDeviceElectroprogonTask::OneDeviceElectroprogonTask(int devId, int revId, int uid, const ElectroprogonParametres &params, const ChannelFinder::DeviceInfo &devInfo) : QObject(nullptr),
    mCurrentDevId(devId),
    mCurrentRevId(revId),
    mIsStopped(true),
    mCurrentUid(uid),
    mParams(params),
    mDevInfo(devInfo)
{    
    mTimer = new QTimer(this);
    mTimerElectroprogon = new QTimer(this);
    mTimerOutRk = new QTimer(this);
    mTimerInRk = new QTimer(this);
    connect(mTimer, &QTimer::timeout, this, &OneDeviceElectroprogonTask::onTimer);
    connect(mTimerElectroprogon, &QTimer::timeout, this, &OneDeviceElectroprogonTask::onTimerElectroprogon);
    connect(mTimerOutRk, &QTimer::timeout, this, &OneDeviceElectroprogonTask::onTimerOutRk);
    connect(mTimerInRk, &QTimer::timeout, this, &OneDeviceElectroprogonTask::onTimerInRk);

    moveToThread(&mThread);
    mThread.start();
}

OneDeviceElectroprogonTask::~OneDeviceElectroprogonTask()
{
    mThread.quit();
    mThread.wait();
}

void OneDeviceElectroprogonTask::onStart()
{
    mFlagRk = true;
	mCurrentDevStat = IWidgetDevice::generateDevStat(mCurrentDevId, mCurrentRevId);
    mIsStopped = false;
    initDevice();
    mTimer->start(cTimerPeriod);
    mTimerElectroprogon->start(cTimerElectroprogonPeriod);
    mTimerOutRk->start(cTimerOutRkPeriod);
    mTimerInRk->start(cTimerInRkPeriod);
}

void OneDeviceElectroprogonTask::onStop()
{  
    QEventLoop loop;
    QTimer waitTimer;
    // ожидание завершения корутин-таймеров
    connect(&waitTimer, &QTimer::timeout, this, [this, &loop](void)->void {
        if (!loop.isRunning())
            return;
        if ((mIsEndTimer) && (mIsEndTimerElectroprogon)) {
            loop.exit();
        }
    });
    // запуск ожидания завершения корутин-таймеров
    waitTimer.start(cTimerElectroprogonPeriod);
    loop.exec();
    mTimer->stop();
    mTimerElectroprogon->stop();
    mTimerOutRk->stop();
    mTimerInRk->stop();

    // вычёрпывание и печать остатков
    onTimerInRk();
    deinitDevice();
    processingTransmitters();
    processingReceivers();
    emit signalSendStatistics(mCurrentUid, mCurrentDevStat);
    mIsStopped = true;
    emit signalHasFinished();
}

void OneDeviceElectroprogonTask::onTimer()
{
    if (mIsStopped)
        return;
    mIsEndTimer = false;
    // отправка наработанной статистики
    emit signalSendStatistics(mCurrentUid, mCurrentDevStat);
    mIsEndTimer = true;
}

void OneDeviceElectroprogonTask::onTimerElectroprogon()
{
    if (mIsStopped)
        return;
    mIsEndTimerElectroprogon = false;
    // обработка передатчиков
    processingTransmitters();
    // обработка приёмников
    processingReceivers();
    mIsEndTimerElectroprogon = true;
}

void OneDeviceElectroprogonTask::onTimerOutRk()
{
    if (mIsStopped)
        return;
    if (!mFlagRk)
        return;
    FDAdapter fd(mDevInfo.mDevName);
    // выдача рк
    IS_EXT_WRAPPER(fd, a429_manageScOut(fd, A429_SC_OUT_1, A429_SC_OUT_1, A429_SC_OUT_1, A429_SC_OUT_1, A429_SC_OUT_1, A429_SC_OUT_1, A429_SC_OUT_1, A429_SC_OUT_1));
    for (auto index = 0; index < mCurrentDevStat.mRkOut.count(); index++) {
        mCurrentDevStat.mRkOut[index]++;
    }
    // sleep(2);
    IS_EXT_WRAPPER(fd, a429_manageScOut(fd, A429_SC_OUT_0, A429_SC_OUT_0, A429_SC_OUT_0, A429_SC_OUT_0, A429_SC_OUT_0, A429_SC_OUT_0, A429_SC_OUT_0, A429_SC_OUT_0));
    IS_EXT_WRAPPER(fd, a429_close(fd));
    mFlagRk = false;
}

void OneDeviceElectroprogonTask::onTimerInRk()
{
    if (mIsStopped)
        return;
    if (mFlagRk)
        return;
    FDAdapter fd(mDevInfo.mDevName);
    // приём рк
    T_CHECK_TRACKING_INT trackCheckInt;
    // проверяем результат ВХ РК
    IS_EXT_WRAPPER(fd, a429_checkTrackingInterrupt(fd, &trackCheckInt));
    auto count = mCurrentDevStat.mRkIn.count();
    for (auto index = 0; index < count; index++) {
        if (trackCheckInt.rkInAppearCount[index])
            mCurrentDevStat.mRkIn[index]++;
    }
    IS_EXT_WRAPPER(fd, a429_close(fd));

    // сброс счётчиков прерываний
    auto receivers = ChannelFinder::getReceiveChannelNameListByNumDevice(mDevInfo.mDevNum);
    foreach (const QString &receiver, receivers)
    {
        FDAdapter fd(receiver);
        T_TRACKING_INT trackInt;
        trackInt.manage = T_TRACKING_INT_MAN_OFF;
        IS_EXT_WRAPPER(fd, a429_manageTrackingInterrupt(fd, &trackInt));
        trackInt.manage = T_TRACKING_INT_MAN_ON;
        IS_EXT_WRAPPER(fd, a429_manageTrackingInterrupt(fd, &trackInt));
        IS_EXT_WRAPPER(fd, a429_close(fd));
    }

    mFlagRk = true;
}

void OneDeviceElectroprogonTask::initDevice()
{
    FDAdapter fdev(mDevInfo.mDevName);
    IS_EXT_WRAPPER(fdev, a429_manageDma(fdev, A429_ON));
    IS_EXT_WRAPPER(fdev, a429_clearDma(fdev));
    IS_EXT_WRAPPER(fdev, a429_close(fdev));

    // инициализация каналов
    auto receivers = ChannelFinder::getReceiveChannelNameListByNumDevice(mDevInfo.mDevNum);
    foreach (const QString &receiver, receivers) {
        FDAdapter fd(receiver);
        IS_EXT_WRAPPER(fd, a429_deinit(fd));
        IS_EXT_WRAPPER(fd, a429_manageChannelDma(fd, A429_ON));
        IS_EXT_WRAPPER(fd, a429_setSpeed(fd, mParams.mSpeed));
        IS_EXT_WRAPPER(fd, a429_parityManage(fd, A429_ON, A429_ON));
        IS_EXT_WRAPPER(fd, a429_manageReverce(fd, A429_OFF));
        IS_EXT_WRAPPER(fd, a429_manageEnBit(fd, A429_ON));

        // IS_EXT_WRAPPER(fd, a429_rxSetReceiveReadyByRkIn(fd, RKIN_NUM_1));

        IS_EXT_WRAPPER(fd, a429_close(fd));
    }
    auto transmitters = ChannelFinder::getTransmitChannelNameListByNumDevice(mDevInfo.mDevNum);
    foreach (const QString &transmitter, transmitters) {
        FDAdapter fd(transmitter);
        IS_EXT_WRAPPER(fd, a429_deinit(fd));
        IS_EXT_WRAPPER(fd, a429_manageEnBit(fd, A429_OFF));
        IS_EXT_WRAPPER(fd, a429_txGapBits(fd, 4));
        IS_EXT_WRAPPER(fd, a429_manageChannelDma(fd, A429_ON));
        IS_EXT_WRAPPER(fd, a429_setSpeed(fd, mParams.mSpeed));
        IS_EXT_WRAPPER(fd, a429_parityManage(fd, A429_ON, A429_ON));
        IS_EXT_WRAPPER(fd, a429_manageReverce(fd, A429_OFF));
        IS_EXT_WRAPPER(fd, a429_txSetMode(fd, A429_MODE_1));
        IS_EXT_WRAPPER(fd, a429_txRrD(fd, mParams.mDiscrRRT));
        IS_EXT_WRAPPER(fd, a429_manageEnBit(fd, A429_ON));
        IS_EXT_WRAPPER(fd, a429_close(fd));
    }
    writeMem();
    // инициализация вых рк
    FDAdapter fd(mDevInfo.mDevName);
    IS_EXT_WRAPPER(fd, a429_manageInterruptMask(fd,
                             1<<8 | 1<<9 | 1<<10 | 1<<11 | 1<<12 | 1<<13 | 1<<14 | 1<<15,
                             1<<8 | 1<<9 | 1<<10 | 1<<11 | 1<<12 | 1<<13 | 1<<14 | 1<<15));
    IS_EXT_WRAPPER(fd, a429_manageScIntMask(fd, A429_IN_RK_1, A429_SC_RISE, A429_ON));
    IS_EXT_WRAPPER(fd, a429_manageScIntMask(fd, A429_IN_RK_2, A429_SC_RISE, A429_ON));
    IS_EXT_WRAPPER(fd, a429_manageScIntMask(fd, A429_IN_RK_3, A429_SC_RISE, A429_ON));
    IS_EXT_WRAPPER(fd, a429_manageScIntMask(fd, A429_IN_RK_4, A429_SC_RISE, A429_ON));
    IS_EXT_WRAPPER(fd, a429_manageScIntMask(fd, A429_IN_RK_5, A429_SC_RISE, A429_ON));
    IS_EXT_WRAPPER(fd, a429_manageScIntMask(fd, A429_IN_RK_6, A429_SC_RISE, A429_ON));
    IS_EXT_WRAPPER(fd, a429_manageScIntMask(fd, A429_IN_RK_7, A429_SC_RISE, A429_ON));
    IS_EXT_WRAPPER(fd, a429_manageScIntMask(fd, A429_IN_RK_8, A429_SC_RISE, A429_ON));
    T_TRACKING_INT trackInt;
    // включаем слежение за РК ВХ
    trackInt.manage = T_TRACKING_INT_MAN_ON;
    IS_EXT_WRAPPER(fd, a429_manageTrackingInterrupt(fd, &trackInt));
    IS_EXT_WRAPPER(fd, a429_close(fd));
    // старт каналов
    foreach (const QString &transmitter, transmitters) {
        FDAdapter fd(transmitter);
        if (mParams.mOnRRT) {
            IS_EXT_WRAPPER(fd, a429_txStartWithRrt(fd, mParams.mValueRRT));
        } else {
            IS_EXT_WRAPPER(fd, a429_txStartStop(fd, A429_START));
        }
        IS_EXT_WRAPPER(fd, a429_close(fd));
    }
}

void OneDeviceElectroprogonTask::deinitDevice()
{
    // останов каналов
    auto transmitters = ChannelFinder::getTransmitChannelNameListByNumDevice(mDevInfo.mDevNum);
    foreach (const QString &transmitter, transmitters) {
        FDAdapter fd(transmitter);
        IS_EXT_WRAPPER(fd, a429_txStartStop(fd, A429_STOP));
        IS_EXT_WRAPPER(fd, a429_close(fd));
    }

    QTimer endTimer;
    QEventLoop endLoop;
    connect(&endTimer, SIGNAL(timeout()), &endLoop, SLOT(quit()));

    endTimer.start(1000);
    endLoop.exec();

    auto receivers = ChannelFinder::getReceiveChannelNameListByNumDevice(mDevInfo.mDevNum);

    auto allChannels = transmitters + receivers;

    // ДЕинициализация каналов
    foreach (const QString &channel, allChannels) {
        FDAdapter fd(channel);
        IS_EXT_WRAPPER(fd, a429_manageEnBit(fd, A429_OFF));
        IS_EXT_WRAPPER(fd, a429_deinit(fd));
        IS_EXT_WRAPPER(fd, a429_close(fd));
    }

    // ДЕинициализация вых рк
    FDAdapter fd(mDevInfo.mDevName);
    T_TRACKING_INT trackInt;
    // включаем слежение за РК ВХ
    trackInt.manage = T_TRACKING_INT_MAN_OFF;
    IS_EXT_WRAPPER(fd, a429_manageTrackingInterrupt(fd, &trackInt));
    IS_EXT_WRAPPER(fd, a429_manageDma(fd, A429_OFF));
    a429_close(fd);
}

void OneDeviceElectroprogonTask::processingTransmitters()
{
    auto transmitters = ChannelFinder::getTransmitChannelNameListByNumDevice(mDevInfo.mDevNum);
    foreach (const QString &transmitter, transmitters) {
        // запросы к каналу
        T_INFO_CHANNEL info;
        T_CONTAINER_DMA dma;
        GlobalTools::readDMA(transmitter, &info, &dma);
        // учёт отправленных сообщений
        mCurrentDevStat.mTMsg[info.numberChannel] += dma.count;
    }
}

void OneDeviceElectroprogonTask::processingReceivers()
{
    auto printErrA429Rcv = [this](T_INFO_CHANNEL* ch, T_CONTAINER_DMA* dma)->void {
        auto hasDmaBlockA429RcvErr = [this](T_BLOCK_DMA* block)->int {
            if (block->word2 & (1<<21)) {
                return 1;
            }
            if (block->word2 & (1<<22)) {
                return 1;
            }
            if (block->word2 & (1<<23)) {
                return 1;
            }
            return 0;
        };
        if (ch->typeChannel == T_INFO_CHANNEL_TYPE_RECIEVER) {
            for (int i=0; i < dma->count; i++) {
                if (hasDmaBlockA429RcvErr( &dma->blocks[i] )) {
                    emit signalPrint(QString("Error in A429 rcv ch %1 dma word2= 0x%2:")
                                     .arg(QString::number(ch->numberChannel))
                                     .arg(QString::number(dma->blocks[i].word2, 16))
                                     );
                    if (dma->blocks[i].word2 & (1<<21)) {
                        emit signalPrint("Ошибка кодировки бита");
                    }
                    if (dma->blocks[i].word2 & (1<<22)) {
                        emit signalPrint("Ошибка паузы");
                    }
                    if (dma->blocks[i].word2 & (1<<23)) {
                        emit signalPrint("Ошибка чётности");
                    }
                }
            }
        }
    };

    auto receivers = ChannelFinder::getReceiveChannelNameListByNumDevice(mDevInfo.mDevNum);
    foreach (const QString &receiver, receivers) {
        // запросы к каналу
        T_INFO_CHANNEL info;
        T_CONTAINER_DMA dma;
        GlobalTools::readDMA(receiver, &info, &dma);
        // учёт отправленных сообщений
        mCurrentDevStat.mRMsg[info.numberChannel] += dma.count;
        // проверка на наличие ошибок и их учёт
        for (auto index=0; index<static_cast<int>(dma.count); index++) {
            if (dma.blocks[index].word2 & ( 1<<21 | 1<<22 | 1<<23)) {
               mCurrentDevStat.mRErr[info.numberChannel] += 1;
            }
        }
        printErrA429Rcv(&info, &dma);
    }
}

bool OneDeviceElectroprogonTask::writeMem()
{
    bool isError = false;
    QVector<MemElement> mem;
    QString memName = QString("%1/mem/%2").arg(QApplication::applicationDirPath()).arg("desk_test_progon.mem");
    auto isReaded = MemLoader::loadMemFile(memName, mem);
    if (!isReaded) {
        isError = true;
        emit signalPrint(QString("dev: '%1' - не удалось считать mem файл: '%2'")
                         .arg(mDevInfo.mDevName)
                         .arg(memName)
                         );
    }
    auto transmitters = ChannelFinder::getTransmitChannelNameListByNumDevice(mDevInfo.mDevNum);
    QVector<MemElement> data;
    for (int i=0; i<256; i++){
        data.append(MemElement(i, 0xAAAAAAAA));
    }
    foreach (QString transmitter, transmitters) {
        bool isWrited;
        isWrited = MemLoader::writeToChannel(transmitter, mem);
        if (!isWrited) {
            isError = true;
            emit signalPrint(QString("dev: '%1' - не удалось загрузить mem файл в передатчик '%2'")
                             .arg(mDevInfo.mDevName)
                             .arg(transmitter)
                             );
        }
        isWrited = MemLoader::writeDataToChannel(transmitter, data);
        if (!isWrited) {
            isError = true;
            emit signalPrint(QString("dev: '%1' - не удалось загрузить данные в передатчик '%2'")
                             .arg(mDevInfo.mDevName)
                             .arg(transmitter)
                             );
        }
    }
    return !isError;
}
