#include "a429_library_adapter.h"
#include <QStringList>
#include <QMutex>
#include <QMutexLocker>

FDAdapter::FDAdapter(const QString &path) : m_path(path)
{
    reconnect();
}

bool FDAdapter::isOk() const
{
    if(m_isExt)
    {
        return m_fd != -1;
    }
    else
    {
        return m_cfd.socketFD != -1;
    }
}

void FDAdapter::reconnect()
{
    static QMutex mutex;
    QMutexLocker lock(&mutex);


    if(m_path.startsWith("/dev"))
    {
        m_isExt = false;

#ifdef HAS_A429_LIBRARY
        m_fd = a429_open(m_path.toLatin1().constData());
#endif
    }
    else
    {
        m_isExt = true;

        static QMap<QString, A429ChannelFD> m_pathToCfd;

        A429ChannelFD cachedFd = m_pathToCfd.value(m_path, A429ChannelFD { -1, 0 });
        if(isInit)
        {
            exta429_close(cachedFd);
            m_pathToCfd.remove(m_path);
            cachedFd.socketFD = -1;
        }

        if(cachedFd.socketFD > 0)
        {
            m_cfd = cachedFd;
            return;
        }

        m_cfd.socketFD = -1;
        m_cfd.channel = 0;


        QStringList parts = m_path.split(':'); // формат ip_адрес:порт:индекс
        switch(parts.count())
        {
            case 3: break;               // три компоненты
            case 2: parts << "0"; break; // устройство, укажем начальный канал
            default: return;
        }

#ifdef HAS_EXTA429_LIBRARY
        m_cfd = exta429_open(parts.first().toLatin1().constData(), parts.at(1).toInt(), parts.at(2).toInt());
#endif

        if(isOk())
            m_pathToCfd[ m_path ] = m_cfd;

        isInit = true;
    }
}
