#ifndef A429_LIBRARY_ADAPTER_H
#define A429_LIBRARY_ADAPTER_H

#ifdef HAS_A429_LIBRARY
#include "a429_library.h"
#endif

#ifdef HAS_EXTA429_LIBRARY
#include "a429ext_library.h"
#endif

#include <QString>
#include <QDebug>

class FDAdapter
{
public:
#ifdef HAS_A429_LIBRARY
    /// Дескриптор символьного устройства (драйвер)
    operator int() { return m_fd; }

protected:
    int m_fd { -1 }; ///< Дескриптор символьного устройства (драйвер)
#endif

#ifdef HAS_EXTA429_LIBRARY
public:
    /// Дескриптор сетевого устройства (сокет)
    operator A429ChannelFD() { return m_cfd; }

protected:
    A429ChannelFD m_cfd { -1, 0 }; ///< Дескриптор сетевого устройства (сокет)
#endif

public:
    /// Открывает устройство по заданному пути (сетевому или символьному)
    FDAdapter(const QString &path);

    /// является ли устройство внешним (сетевым)
    bool isExt() const { return m_isExt; }

    bool isOk() const;

    void reconnect();

protected:
    bool m_isExt;
    QString m_path;
    bool isInit { false };
};


#if defined HAS_A429_LIBRARY && defined HAS_EXTA429_LIBRARY

 /**
  \brief обёртка для точно такого же вызова, но с приставкой ext, для внешних устройств
  \param X признак того, что устройство внешнее
  \param Y команда для выполнения
 */
#if 1

#define IS_EXT_WRAPPER(fd, Y) [&]{ \
    int ret = 0; \
    ret = (fd.isExt()) ? ext##Y : Y; \
    if(ret != 0) { \
        qDebug() << #Y " returned " << ret; \
    } \
    return ret; }();

#else

#define IS_EXT_WRAPPER(fd, Y) [&]{ \
    int ret = 0; \
    do { \
        ret = (fd.isExt()) ? ext##Y : Y; \
        if(ret == -19) { \
            qDebug() << "ret == -19"; \
            if(!fd.isOk()) \
            fd.reconnect(); \
        } \
    } \
    while(ret == -19); \
    assert(ret == 0); \
    return ret; }();

#endif

#endif

#if defined HAS_A429_LIBRARY && !defined HAS_EXTA429_LIBRARY

/**
 \brief обёртка для точно такого же вызова, но с приставкой ext, для внешних устройств
 \param X признак того, что устройство внешнее (не используется)
 \param Y команда для выполнения - всегда через библиотеку драйвера
*/
#define IS_EXT_WRAPPER(X, Y) Y
#endif

#if (!defined HAS_A429_LIBRARY) && defined HAS_EXTA429_LIBRARY

 /**
  \brief обёртка для точно такого же вызова, но с приставкой ext, для внешних устройств
  \param X признак того, что устройство внешнее
  \param Y команда для выполнения - всегда через библиотеку сокетов
 */
#define IS_EXT_WRAPPER(X, Y) ext##Y
#endif

#endif // A429_LIBRARY_ADAPTER_H
