#include <QDir>
#include <QRegExp>
#include <QRegularExpression>

#include "channelfinder.h"

QList<Device> g_devices;

bool ChannelFinder::sortCondition(const QString &a, const QString &b)
{
    auto listA = a.split('-');
    auto listB = b.split('-');

    if (static_cast<QString>(listA.at(1)).toInt() == static_cast<QString>(listB.at(1)).toInt())
        return static_cast<QString>(listA.at(3)).toInt() < static_cast<QString>(listB.at(3)).toInt();
    else
        return static_cast<QString>(listA.at(1)).toInt() < static_cast<QString>(listB.at(1)).toInt() ;
}

QStringList ChannelFinder::getChannelNameList()
{
    QStringList result;

    for(const Device &dev : g_devices)
        result += dev.allChannels;

    return result;
}

void ChannelFinder::resetDeviceNameList()
{
    QDir dev("/dev");
    QStringList allFiles = dev.entryList(QDir::System);
    QRegExp rx("a429.*-(\\d+)-ch-(\\d+)");
    // обыскать каталог dev и найти паттерны
    QStringList channels;
    foreach (const QString &fileName, allFiles) {
        if (rx.exactMatch(fileName)) {
            channels.append(QString("/dev/%1").arg(fileName));
        }
    }
    qSort(channels.begin(), channels.end(), sortCondition);

    // удаляем локальные устройства
    auto it = g_devices.begin();
    while(it != g_devices.end() && !it->isExt)
        it++;
    g_devices.erase(g_devices.begin(), it);

    int devCount = 0;
    foreach (const QString &deviceName, channels)
    {
        if (!deviceName.endsWith("-ch-0"))
            continue;

        Device dev;
        dev.isExt = false;
        dev.path = deviceName;

        FDAdapter fd(deviceName);
        auto ret = IS_EXT_WRAPPER(fd, a429_getDevInfoChannels(fd, &(dev.info_ch)));
        IS_EXT_WRAPPER(fd, a429_close(fd));

        if(ret != GOOD)
            continue;

        QString prefix = QString(deviceName).remove(QRegularExpression("-ch-0$"));

        foreach (const QString &channelName, channels)
        {
            if (!channelName.startsWith(prefix))
                continue;

            FDAdapter fd(channelName);
            T_INFO_CHANNEL info_channel;
            auto ret = IS_EXT_WRAPPER(fd, a429_getChannelInfo(fd, &info_channel));
            IS_EXT_WRAPPER(fd, a429_close(fd));

            if(ret != GOOD)
                continue;

            if(info_channel.typeChannel == T_INFO_CHANNEL_TYPE_RECIEVER)
            {
                dev.receiveChannels += channelName;
            }
            else
            {
                if(info_channel.typeChannel == T_INFO_CHANNEL_TYPE_TRANSMITTER)
                    dev.transmitChannels += channelName;
                else
                    continue;
            }

            dev.allChannels += channelName;
        }

        if(dev.info_ch.countRecieverChannels == dev.receiveChannels.count() &&
                dev.info_ch.countTransmitterChannels == dev.transmitChannels.count())
            g_devices.insert(devCount++, dev);
    }
}

QStringList ChannelFinder::getDeviceNameList()
{
    QStringList result;

    for(const Device &dev : g_devices)
        result << dev.path;

    return result;
}

QVector<ChannelFinder::DeviceInfo> ChannelFinder::getDeviceInfoList()
{
    auto devNames = ChannelFinder::getDeviceNameList();
    QVector<ChannelFinder::DeviceInfo> devices;
    for(const QString &devName : devNames)
    {
        ChannelFinder::DeviceInfo dev;
        dev.mDevName = devName;
        dev.mDevNum = devices.size();
        devices.append(dev);
    }
    return devices;
}

int ChannelFinder::getCountDevices()
{
    return g_devices.count();
}

QStringList ChannelFinder::getChannelNameListByNumDevice(int numDev)
{
    return g_devices.at(numDev).allChannels;
}

QStringList ChannelFinder::getReceiveChannelNameListByNumDevice(int numDev)
{
    return g_devices.at(numDev).receiveChannels;
}

QStringList ChannelFinder::getTransmitChannelNameListByNumDevice(int numDev)
{
    return g_devices.at(numDev).transmitChannels;
}

QStringList ChannelFinder::getNetworkDevices()
{
    QStringList result;

    for(const Device &dev : g_devices)
        if(dev.isExt)
            result << dev.path;

    return result;
}

bool setNetworkDevice(Device *destDevice, const ExtDevice &device)
{
#ifndef HAS_EXTA429_LIBRARY
    return false;
#else
    FDAdapter fd(device + ":0");

    Device dev;
    dev.isExt = true;
    dev.path = device;
    auto ret = IS_EXT_WRAPPER(fd, a429_getDevInfoChannels(fd, &(dev.info_ch)));
    IS_EXT_WRAPPER(fd, a429_close(fd));

    if(ret != GOOD)
        return false;

    for(int channelId = 0; channelId < dev.info_ch.countRecieverChannels + dev.info_ch.countTransmitterChannels; ++channelId)
    {
        QString channelName = dev.path + ":" + QString::number(channelId);

        FDAdapter fd(channelName);
        T_INFO_CHANNEL info_channel;
        auto ret = IS_EXT_WRAPPER(fd, a429_getChannelInfo(fd, &info_channel));
        IS_EXT_WRAPPER(fd, a429_close(fd));

        if(ret != GOOD)
            break;

        if(info_channel.typeChannel == T_INFO_CHANNEL_TYPE_RECIEVER)
        {
            dev.receiveChannels += channelName;
        }
        else
        {
            if(info_channel.typeChannel == T_INFO_CHANNEL_TYPE_TRANSMITTER)
                dev.transmitChannels += channelName;
            else
                break;
        }

        dev.allChannels += channelName;
    }


    if(dev.info_ch.countRecieverChannels == dev.receiveChannels.count() &&
            dev.info_ch.countTransmitterChannels == dev.transmitChannels.count())
    {
        *destDevice = dev;
        return true;
    }

    return false;
#endif
}

bool ChannelFinder::addNetworkDevice(const ExtDevice &device)
{
    Device dev;
    if(!setNetworkDevice(&dev, device))
        return false;

    g_devices.append(dev);
    return true;
}

bool ChannelFinder::editNetworkDevice(int index, const ExtDevice &device)
{
    // ищем индекс сетевого устройства - пропускаем локальные
    auto it = g_devices.begin();
    while(it != g_devices.end() && !it->isExt)
        it++;

    for(int i = 0; it != g_devices.end() && i < index; ++i)
        it++;

    if(it == g_devices.end())
        return false; // ошибка индекса

    return setNetworkDevice(&(*it), device);
}

void ChannelFinder::removeNetworkDevice(int index)
{
    // ищем индекс сетевого устройства - пропускаем локальные
    auto it = g_devices.begin();
    while(it != g_devices.end() && !it->isExt)
        it++;

    for(int i = 0; it != g_devices.end() && i < index; ++i)
        it++;

    if(it == g_devices.end())
        return; // ошибка индекса

    g_devices.erase(it);
}
