#ifndef CHANNEL_FINDER_H
#define CHANNEL_FINDER_H

#include <QObject>
#include <QStringList>
#include <QVector>
#include "a429_library_adapter.h"

struct Device
{
    bool isExt;
    QString path;
    QString location;

    T_INFO_DEVICE info_ch;
    QStringList allChannels;

    QStringList receiveChannels;
    QStringList transmitChannels;
};

struct ExtDevice
{
    QString ipAddress;
    int port;

    operator QString() const { return ipAddress + ":" + QString::number(port); }
};

///
/// \brief The ChannelFinder class поисковик каналов
///
class ChannelFinder
{
private:
    ///
    /// \brief sortCondition условие сортировки каналов плат
    /// \param a текущий
    /// \param b следующий
    /// \return результат
    ///
    static bool sortCondition(const QString& a, const QString& b);
public:
    ///
    /// \brief The ChannelInfo struct информация о канале
    ///
    struct ChannelInfo {
        QString m_name;             ///< полное имя канала
        QString m_pciLocation;      ///< информация об адресе pci
    };
    ///
    /// \brief The DeviceInfo struct информация о плате
    ///
    struct DeviceInfo {
        QString mDevName;           ///< имя платы
        int mDevNum;                ///< номер платы
    };

    /// \brief Опрос файловой системы для получения списка каналов и устройст
    static void resetDeviceNameList();

    /// \brief getChannelNameList получить список имён каналов
    /// \return список имён каналов
    static QStringList getChannelNameList();
    /// \brief getDeviceNameList получить список плат
    /// \return список плат
    static QStringList getDeviceNameList();
    /// \brief getDeviceInfoList получить список плат
    /// \return список плат
    static QVector<DeviceInfo> getDeviceInfoList();
    ///
    /// \brief getCountDevices получить кол-во девайсов
    /// \return кол-во девайсов
    ///
    static int getCountDevices();
    ///
    /// \brief getChannelNameListByNumDevice получить список каналов по номеру платы
    /// \param numDev номер платы
    /// \return список каналов
    ///
    static QStringList getChannelNameListByNumDevice(int numDev);
    ///
    /// \brief getReceiveChannelNameListByNumDevice получить список приёмников платы
    /// \param numDev номер девайса
    /// \return список каналов
    ///
    static QStringList getReceiveChannelNameListByNumDevice(int numDev);
    ///
    /// \brief getTransmitChannelNameListByNumDevice получить список передатчиков платы
    /// \param numDev номер девайса
    /// \return список каналов
    ///
    static QStringList getTransmitChannelNameListByNumDevice(int numDev);

    ///
    /// \brief Получить настроенные сетевые устройства
    /// \return сетевые устройства
    static QStringList getNetworkDevices();

    ///
    /// \brief Получить настроенные сетевые устройства
    /// \return сетевые устройства
    static bool addNetworkDevice(const ExtDevice &device);

    ///
    /// \brief Получить настроенные сетевые устройства
    /// \return сетевые устройства
    static bool editNetworkDevice(int index, const ExtDevice &device);

    ///
    /// \brief Получить настроенные сетевые устройства
    /// \return сетевые устройства
    static void removeNetworkDevice(int index);
};

#endif // CHANNEL_FINDER_H
