#include "globaltools.h"
#include <QTime>

quint32 GlobalTools::readRegRxConf(const QString &channelName)
{
    quint32 result = 0xFFFFFFFF;

    T_INFO_CHANNEL info;
    FDAdapter fd(channelName);

    IS_EXT_WRAPPER(fd, a429_getChannelInfo(fd, &info));
    if (info.typeChannel != T_INFO_CHANNEL_TYPE_TRANSMITTER)
    {
        SADDR_DATA reg;
        reg.daddr = REG_RX_CONF(info.numberChannel);
        IS_EXT_WRAPPER(fd, a429_readReg(fd, &reg));
        result = reg.data;
    }

    IS_EXT_WRAPPER(fd, a429_close(fd));
    return result;
}

quint32 GlobalTools::readRegTxConf(const QString &channelName)
{
    quint32 result = 0xFFFFFFFF;

    T_INFO_CHANNEL info;
    FDAdapter fd(channelName);

    IS_EXT_WRAPPER(fd, a429_getChannelInfo(fd, &info));
    if (info.typeChannel != T_INFO_CHANNEL_TYPE_RECIEVER)
    {
        SADDR_DATA reg;
        reg.daddr = REG_TX_CONF(info.numberChannel);
        IS_EXT_WRAPPER(fd, a429_readReg(fd, &reg));
        result = reg.data;
    }

    IS_EXT_WRAPPER(fd, a429_close(fd));
    return result;
}

quint32 GlobalTools::readRegTxControl(const QString &channelName)
{
    quint32 result = 0xFFFFFFFF;

    T_INFO_CHANNEL info;
    FDAdapter fd(channelName);

    IS_EXT_WRAPPER(fd, a429_getChannelInfo(fd, &info));
    if (info.typeChannel != T_INFO_CHANNEL_TYPE_RECIEVER)
    {
        SADDR_DATA reg;
        reg.daddr = REG_TX_CONTROL(info.numberChannel);
        IS_EXT_WRAPPER(fd, a429_readReg(fd, &reg));
        result = reg.data;
    }

    IS_EXT_WRAPPER(fd, a429_close(fd));
    return result;
}

QString GlobalTools::convertValToBinStr(quint32 value)
{
    auto line = QString::number(value, 2).rightJustified(32, '0').toUpper();
    auto resultLine = QString("%1 %2 %3 %4")
            .arg(line.mid(0, 8))
            .arg(line.mid(8, 8))
            .arg(line.mid(16, 8))
            .arg(line.mid(24, 8));
    return resultLine;
}

QString GlobalTools::convertValToHexStr(quint32 value)
{
    auto line = QString::number(value, 16).rightJustified(8, '0').toUpper();
    auto resultLine = QString("0x%1")
            .arg(line);
    return resultLine;
}

int GlobalTools::readRegRxLabelConf(const QString &channelName, int label, int &numReg, quint32 &valReg)
{
    unsigned int ret = -1;
    SADDR_DATA reg {0, 0};
    T_INFO_CHANNEL chInfo;
    FDAdapter fd(channelName);
    numReg = -1;
    valReg = 0;
    IS_EXT_WRAPPER(fd, a429_getChannelInfo(fd, &chInfo));

    if (chInfo.typeChannel != T_INFO_CHANNEL_TYPE_TRANSMITTER && label >= 0 && label < 256)
    {
        numReg = label / 32;
        switch(numReg)
        {
            case 0: reg.daddr = REG_LBL_CONF_PCI_0(chInfo.numberChannel); break;
            case 1: reg.daddr = REG_LBL_CONF_PCI_1(chInfo.numberChannel); break;
            case 2: reg.daddr = REG_LBL_CONF_PCI_2(chInfo.numberChannel); break;
            case 3: reg.daddr = REG_LBL_CONF_PCI_3(chInfo.numberChannel); break;
            case 4: reg.daddr = REG_LBL_CONF_PCI_4(chInfo.numberChannel); break;
            case 5: reg.daddr = REG_LBL_CONF_PCI_5(chInfo.numberChannel); break;
            case 6: reg.daddr = REG_LBL_CONF_PCI_6(chInfo.numberChannel); break;
            case 7: reg.daddr = REG_LBL_CONF_PCI_7(chInfo.numberChannel); break;
            default: break;
        }
    }

    if(reg.daddr != 0)
    {
        ret = IS_EXT_WRAPPER(fd, a429_readReg(fd, &reg));
        valReg = reg.data;
    }

    IS_EXT_WRAPPER(fd, a429_close(fd));
    return ret;
}

quint32 GlobalTools::mirroredLabel(quint32 data)
{
    quint32 mirroredData = 0;
    int shift = 7;
    for (int index = 0; index < 8; index++, shift--) {
        mirroredData |= (data & (1<<index)) ? (1 << shift) : 0;
    }
    mirroredData |= (data & 0xFFFFFF00);
    return mirroredData;
}

quint32 GlobalTools::generateMask(int firstBit, int lastBit)
{
    quint32 mask = 0;
    for (int shift = firstBit; shift <= lastBit; shift++) {
        mask |= 1<<shift;
    }
    return mask;
}

quint32 GlobalTools::clearAndManageDMA(const QString &devName, int dmaMode)
{
    FDAdapter fd(devName);

    IS_EXT_WRAPPER(fd, a429_clearDma(fd));
    auto result = IS_EXT_WRAPPER(fd, a429_manageDma(fd, dmaMode));

    IS_EXT_WRAPPER(fd, a429_close(fd));
    return result;
}

quint32 GlobalTools::readDMA(const QString &channelName, T_INFO_CHANNEL *info, T_CONTAINER_DMA *container)
{
    // запросы к каналу
    FDAdapter fd(channelName);
    IS_EXT_WRAPPER(fd, a429_getChannelInfo(fd, info));
    container->count = 0;
    auto result = IS_EXT_WRAPPER(fd, a429_readDma(fd, container));

    static const QMap<int, QString> regToString
    {{REG_DMA_DATA_BASE, "REG_DMA_DATA_BASE"}, { REG_DMA_DISABLE, "REG_DMA_DISABLE"}, {REG_INTERRUPT, "REG_INTERRUPT"}, { REG_DMA_INDEX, "REG_DMA_INDEX"}, { REG_DATA_COUNTER, "REG_DATA_COUNTER"},};

    QString log;
    for(auto it = regToString.begin(); it != regToString.end(); ++it)
    {
        SADDR_DATA reg;
        reg.daddr = it.key(); // указатель записи в буфере данных
        reg.data = 0;
        IS_EXT_WRAPPER(fd, a429_readReg(fd, &reg));

        log += ", " + it.value() + ": " + QString::number(reg.data, 16);
    }
    qDebug() << channelName << QTime::currentTime().toString(Qt::ISODateWithMs) << qPrintable(log);

    IS_EXT_WRAPPER(fd, a429_close(fd));
    return result;
}

quint32 GlobalTools::writeRAM(const QString &channelName, RAM_BLOCK *ram)
{
    FDAdapter fd(channelName);

    auto result = IS_EXT_WRAPPER(fd, a429_writeRam(fd, ram));

    IS_EXT_WRAPPER(fd, a429_close(fd));
    return result;
}
