#ifndef GLOBALTOOLS_H
#define GLOBALTOOLS_H

#include <QObject>


#include "a429_library_adapter.h"

///
/// \brief The GlobalTools class глобальные инструменты
///
class GlobalTools
{
public:
    /// очистка и управление DMA
    static quint32 clearAndManageDMA(const QString &devName, int dmaMode);

    /// чтение dma
    static quint32 readDMA(const QString &channelName, T_INFO_CHANNEL *info, T_CONTAINER_DMA *container);

    static quint32 writeRAM(const QString &channelName, RAM_BLOCK *ram);

    ///
    /// \brief readRegRxConf прочитать регистр RX_CONF
    /// \param channelName имя канала
    /// \return значение регистра
    ///
    static quint32 readRegRxConf(const QString& channelName);
    ///
    /// \brief readRegTxConf прочитать регистр TX_CONF
    /// \param channelName имя канала
    /// \return значение регистра
    ///
    static quint32 readRegTxConf(const QString& channelName);
    ///
    /// \brief readRegTxControl прочитать регистр TX_CONTROL
    /// \param channelName имя канала
    /// \return значение регистра
    ///
    static quint32 readRegTxControl(const QString& channelName);
    ///
    /// \brief convertU32toBinString перевод числа в двоичный текстовый вид
    /// \param value число
    /// \return двоичный текстовый вид
    ///
    static QString convertValToBinStr(quint32 value);
    ///
    /// \brief convertValToHexStr перевод числа в шестнадцатеричный текстовый вид
    /// \param value число
    /// \return шестнадцатеричный текстовый вид
    ///
    static QString convertValToHexStr(quint32 value);
    ///
    /// \brief readRegRxLabelConf Чтение регистра фильтра, соотв. метке
    /// \param channelName имя канала
    /// \param label метка
    /// \param numReg номер регистра фильтра
    /// \param valReg значение регистра
    /// \return код результата
    ///
    static int readRegRxLabelConf(const QString& channelName, int label, int& numReg, quint32& valReg);
    ///
    /// \brief mirroredLabel отзеркалить метку (адрес)
    /// \param data слово дпк
    /// \return отзеркаленная метка
    ///
    static quint32 mirroredLabel(quint32 data);
    ///
    /// \brief generateMask генерирует битовую маску
    /// \param firstBit начальный бит
    /// \param lastBit конечный бит
    /// \return маска
    ///
    static quint32 generateMask(int firstBit, int lastBit);
};

#endif // GLOBALTOOLS_H
