#include <QFile>
#include <QTextStream>
#include <QStringList>

#include "a429_library_adapter.h"
#include "memloader.h"

bool MemLoader::loadMemFile(const QString &fullname, QVector<MemElement> &mem)
{
    bool result = false;
    QFile file(fullname);
    if (file.exists()) {
        if (file.open(QFile::ReadOnly)) {
            mem.clear();
            QTextStream stream(&file);
            while (!stream.atEnd()) {
                auto line = stream.readLine();
                QStringList tokens = line.split(":");
                int numWord = ((QString)tokens.first()).toInt();
                unsigned int word = ((QString)tokens.last()).toUInt(nullptr, 16);
                MemElement element(numWord, word);
                mem.append(element);
            }
            result = true;
            file.close();
        }
    }
    return result;
}

bool MemLoader::writeToChannel(const QString &channelName, const QVector<MemElement> &mem)
{
    int err;
    bool result = false;
    FDAdapter fd(channelName);
    T_INFO_CHANNEL info;
    IS_EXT_WRAPPER(fd, a429_getChannelInfo(fd, &info));
    if (info.typeChannel == T_INFO_CHANNEL_TYPE_RECIEVER)
    {
    }
    else
    {
        RAM_BLOCK ram, ramRead;
        ram.type = RAM_BLOCK_TYPE_DESCRIPTOR;
        ram.shift = 0;
        ram.length = mem.count();
        ram.dwords = new unsigned int[ram.length]();
        foreach( MemElement element, mem )
            ram.dwords[element.mNumWord] = element.mWord;

        err = IS_EXT_WRAPPER(fd, a429_writeRam(fd, &ram));
        ramRead.type = RAM_BLOCK_TYPE_DESCRIPTOR;
        ramRead.shift = 0;
        ramRead.length = ram.length;
        ramRead.dwords = new unsigned int[ramRead.length]();
        IS_EXT_WRAPPER(fd, a429_readRam(fd, &ramRead));
        for (int i = 0; i < ram.length; i++)
            if (ram.dwords[i] != ramRead.dwords[i])
            {
                err = -1;
                break;
            }
        delete[] ramRead.dwords;
        delete[] ram.dwords;
    }

    IS_EXT_WRAPPER(fd, a429_close(fd));

    if (err == 0)
        result = true;
    else
        result = false;

    return result;
}

bool MemLoader::writeDataToChannel(const QString &channelName, const QVector<MemElement> &mem)
{
    int err;
    bool result = false;
    FDAdapter fd(channelName);
    T_INFO_CHANNEL info;
    IS_EXT_WRAPPER(fd, a429_getChannelInfo(fd, &info));
    if (info.typeChannel == T_INFO_CHANNEL_TYPE_RECIEVER)
    {
    }
    else
    {
        RAM_BLOCK ram, ramRead;
        ram.type = RAM_BLOCK_TYPE_DATA;
        ram.shift = 0;
        ram.length = mem.count();
        ram.dwords = new unsigned int[ram.length]();
        foreach( MemElement element, mem )
            ram.dwords[element.mNumWord] = element.mWord;

        err = IS_EXT_WRAPPER(fd, a429_writeRam(fd, &ram));
        ramRead.type = RAM_BLOCK_TYPE_DATA;
        ramRead.shift = 0;
        ramRead.length = ram.length;
        ramRead.dwords = new unsigned int[ramRead.length]();
        IS_EXT_WRAPPER(fd, a429_readRam(fd, &ramRead));
        for (int i = 0; i<ram.length; i++)
            if (ram.dwords[i] != ramRead.dwords[i])
            {
                err = -1;
                break;
            }

        delete[] ramRead.dwords;
        delete[] ram.dwords;
    }
    IS_EXT_WRAPPER(fd, a429_close(fd));

    if (err == 0)
        result = true;
    else
        result = false;

    return result;
}

MemElement::MemElement(int numWord, unsigned int word):
    mNumWord(numWord), mWord(word)
{
}
