#include <QRegularExpression>
#include "widgetdevice.h"
#include "ui_widgetdevice.h"
#include "a429_library_adapter.h"

WidgetDevice::WidgetDevice(QWidget *parent) :
    QFrame(parent), IWidgetDevice(),
    ui(new Ui::WidgetDevice)
{
    ui->setupUi(this);
    setBackClr(mCurBackClr);
    connect(ui->pushButtonShowDev, &QPushButton::clicked, this, [this](void)->void {
        emit pressedKeyBlinked(mKey);
    });
}

WidgetDevice::~WidgetDevice()
{
    delete ui;
}

#define A429_MAX_CNT_CH 16
#define A429_MAX_CNT_RKIN 8
#define A429_MAX_CNT_RKOUT 8

void WidgetDevice::setVisibleView()
{
    // отобразить статистику передатчиков
    auto listT = this->findChildren<QLabel*>(QRegularExpression("labelT(\\d+)"));
    auto listTMsg = this->findChildren<QLabel*>(QRegularExpression("labelT(\\d+)Msg"));
    auto listTErr = this->findChildren<QLabel*>(QRegularExpression("labelT(\\d+)Err"));
    // отобразить статистику приёмников
    auto listR = this->findChildren<QLabel*>(QRegularExpression("labelR(\\d+)"));
    auto listRMsg = this->findChildren<QLabel*>(QRegularExpression("labelR(\\d+)Msg"));
    auto listRErr = this->findChildren<QLabel*>(QRegularExpression("labelR(\\d+)Err"));
    // отобразить статистику рк
    auto listRkIn = this->findChildren<QLabel*>(QRegularExpression("labelRKIN(\\d+)Cnt"));
    auto listRkInlb = this->findChildren<QLabel*>(QRegularExpression("labelRKIN(\\d+)"));
    auto listRkOut = this->findChildren<QLabel*>(QRegularExpression("labelRKOUT(\\d+)Cnt"));
    auto listRkOutlb = this->findChildren<QLabel*>(QRegularExpression("labelRKOUT(\\d+)"));
    for (int index = 0; index < A429_MAX_CNT_CH; index++) {
        listT[index]->setVisible(true);
        listTMsg[index]->setVisible(true);
        listTErr[index]->setVisible(true);
        listR[index]->setVisible(true);
        listRMsg[index]->setVisible(true);
        listRErr[index]->setVisible(true);
    }
    for (int index = 0; index < A429_MAX_CNT_RKIN; index++) {
        listRkIn[index]->setVisible(true);
        listRkInlb[index]->setVisible(true);
    }
    for (int index = 0; index < A429_MAX_CNT_RKOUT; index++) {
        listRkOut[index]->setVisible(true);
        listRkOutlb[index]->setVisible(true);
    }
}

void WidgetDevice::setUnusedUnVisibleView()
{
    // отобразить статистику передатчиков
    auto listT = this->findChildren<QLabel*>(QRegularExpression("labelT(\\d+)"));
    auto listTMsg = this->findChildren<QLabel*>(QRegularExpression("labelT(\\d+)Msg"));
    auto listTErr = this->findChildren<QLabel*>(QRegularExpression("labelT(\\d+)Err"));
    // отобразить статистику приёмников
    auto listR = this->findChildren<QLabel*>(QRegularExpression("labelR(\\d+)"));
    auto listRMsg = this->findChildren<QLabel*>(QRegularExpression("labelR(\\d+)Msg"));
    auto listRErr = this->findChildren<QLabel*>(QRegularExpression("labelR(\\d+)Err"));
    // отобразить статистику рк
    auto listRkIn = this->findChildren<QLabel*>(QRegularExpression("labelRKIN(\\d+)Cnt"));
    auto listRkInlb = this->findChildren<QLabel*>(QRegularExpression("labelRKIN(\\d+)"));
    auto listRkOut = this->findChildren<QLabel*>(QRegularExpression("labelRKOUT(\\d+)Cnt"));
    auto listRkOutlb = this->findChildren<QLabel*>(QRegularExpression("labelRKOUT(\\d+)"));
    auto devStatParams = IWidgetDevice::getDevStatParametres(mCurrentDevId, mCurrentRevId);
    for (int index = 0; index < A429_MAX_CNT_CH; index++) {
        bool flag = (index >= devStatParams.mCountT) ? false : true;
        listT[index]->setVisible(flag);
        listTMsg[index]->setVisible(flag);
        listTErr[index]->setVisible(flag);
    }
    for (int index = 0; index < A429_MAX_CNT_CH; index++) {
        bool flag = (index >= devStatParams.mCountR) ? false : true;
        listR[index]->setVisible(flag);
        listRMsg[index]->setVisible(flag);
        listRErr[index]->setVisible(flag);
    }
    for (int index = 0; index < A429_MAX_CNT_RKIN; index++) {
        bool flag = (index >= devStatParams.mCountRkIn) ? false : true;
        listRkIn[index]->setVisible(flag);
        listRkInlb[index]->setVisible(flag);
    }
    for (int index = 0; index < A429_MAX_CNT_RKOUT; index++) {
        bool flag = (index >= devStatParams.mCountRkOut) ? false : true;
        listRkOut[index]->setVisible(flag);
        listRkOutlb[index]->setVisible(flag);
    }
}


void WidgetDevice::setStatistics(const DeviceStatistics &devStat)
{
    const DeviceStatisticsParametres& devStatParam = IWidgetDevice::getDevStatParametres(mCurrentDevId, mCurrentRevId);
    // проверка длины наработанной статистики платы
    if (devStat.mTMsg.count() != devStatParam.mCountT) {
        qDebug("%s", "bad mTMsg");
        return;
    }
    if (devStat.mTErr.count() != devStatParam.mCountT) {
        qDebug("%s", "bad mTErr");
        return;
    }
    if (devStat.mRMsg.count() != devStatParam.mCountR) {
        qDebug("%s", "bad mRMsg");
        return;
    }
    if (devStat.mRErr.count() != devStatParam.mCountR) {
        qDebug("%s", "bad mRErr");
        return;
    }
    if (devStat.mRkIn.count() != devStatParam.mCountRkIn) {
        qDebug("%s", "bad mRkIn");
        return;
    }
    if (devStat.mRkOut.count() != devStatParam.mCountRkOut) {
        qDebug("%s", "bad mRkOut");
        return;
    }
    mCurrentStatictics = devStat;
    updateStatistics(devStat);
}

void WidgetDevice::updateStatistics(const DeviceStatistics &devStat)
{
    // отобразить статистику передатчиков
    auto listT = this->findChildren<QLabel*>(QRegularExpression("labelT(\\d+)"));
    auto listTMsg = this->findChildren<QLabel*>(QRegularExpression("labelT(\\d+)Msg"));
    auto listTErr = this->findChildren<QLabel*>(QRegularExpression("labelT(\\d+)Err"));
    for (int index=0; index < devStat.mTMsg.count(); index++) {
        listTMsg[index]->setText(QString::number(devStat.mTMsg[index]));
        listTMsg[index]->setStyleSheet(QString("background-color: %1").arg( ((index % 2)==0) ? "white" : "gray"));
        if ((devStat.mTMsg[index] > 0) && (devStat.mTErr[index] == 0))
            listTMsg[index]->setStyleSheet(QString("background-color: green"));
        listTErr[index]->setText(QString::number(devStat.mTErr[index]));
        listTErr[index]->setStyleSheet(QString("background-color: %1").arg( ((index % 2)==0) ? "white" : "gray"));
        if (devStat.mTErr[index] > 0)
            listTErr[index]->setStyleSheet(QString("background-color: red"));
        listT[index]->setStyleSheet(QString("background-color: %1").arg( ((index % 2)==0) ? "white" : "gray"));
    }
    // отобразить статистику приёмников
    auto listR = this->findChildren<QLabel*>(QRegularExpression("labelR(\\d+)"));
    auto listRMsg = this->findChildren<QLabel*>(QRegularExpression("labelR(\\d+)Msg"));
    auto listRErr = this->findChildren<QLabel*>(QRegularExpression("labelR(\\d+)Err"));
    for (int index=0; index < devStat.mRMsg.count(); index++) {
        listRMsg[index]->setText(QString::number(devStat.mRMsg[index]));
        listRMsg[index]->setStyleSheet(QString("background-color: %1").arg( ((index % 2)==0) ? "white" : "gray"));
        if ((devStat.mRMsg[index] > 0) && (devStat.mRErr[index] == 0))
            listRMsg[index]->setStyleSheet(QString("background-color: green"));
        listRErr[index]->setText(QString::number(devStat.mRErr[index]));
        listRErr[index]->setStyleSheet(QString("background-color: %1").arg( ((index % 2)==0) ? "white" : "gray"));
        if (devStat.mRErr[index] > 0)
            listRErr[index]->setStyleSheet(QString("background-color: red"));
        listR[index]->setStyleSheet(QString("background-color: %1").arg( ((index % 2)==0) ? "white" : "gray"));
    }
    // отобразить статистику рк
    auto listRkIn = this->findChildren<QLabel*>(QRegularExpression("labelRKIN(\\d+)Cnt"));
    auto listRkInlb = this->findChildren<QLabel*>(QRegularExpression("labelRKIN(\\d+)"));
    for (int index=0; index < devStat.mRkIn.count(); index++) {
        listRkIn[index]->setText(QString::number(devStat.mRkIn[index]));
        listRkIn[index]->setStyleSheet(QString("background-color: %1").arg( ((index % 2)==0) ? "white" : "gray"));
        listRkInlb[index]->setStyleSheet(QString("background-color: %1").arg( ((index % 2)==0) ? "white" : "gray"));
    }
    auto listRkOut = this->findChildren<QLabel*>(QRegularExpression("labelRKOUT(\\d+)Cnt"));
    auto listRkOutlb = this->findChildren<QLabel*>(QRegularExpression("labelRKOUT(\\d+)"));
    for (int index=0; index < devStat.mRkOut.count(); index++) {
        listRkOut[index]->setText(QString::number(devStat.mRkOut[index]));
        listRkOut[index]->setStyleSheet(QString("background-color: %1").arg( ((index % 2)==0) ? "white" : "gray"));
        listRkOutlb[index]->setStyleSheet(QString("background-color: %1").arg( ((index % 2)==0) ? "white" : "gray"));
    }
}

void WidgetDevice::setBackClr(WidgetDevice::BACKCLR value)
{
    mCurBackClr = value;
    this->setStyleSheet(QString("background-color: %1").arg( (mCurBackClr == BACKCLR::dark)  ? "#A9A9A9" : "#FFFAFA"));
}

QString WidgetDevice::getHtmlText()
{
    auto transmitTable = [this](void)->QString {
        QString html;
        int cnt = mCurrentStatictics.mTMsg.count();
        html += QString("<table align=\"center\" width=\"100%\" border=\"1\" cellpadding=\"0\" cellspacing=\"0\" >");
        html += QString("<tr>");
        html += QString("<td align=\"left\"> %1 </td>").arg("ПЕРЕДАТЧИКИ:");
        for (int i=0; i<cnt; i++) {
            html += QString("<td align=\"center\"> ПРД%1 </td>").arg(QString::number(i+1));
        }
        html += QString("</tr>");
        html += QString("<tr>");
        html += QString("<td align=\"left\"> %1 </td>").arg("СООБЩЕНИЯ:");
        for (int i=0; i<cnt; i++) {
            QString color;
            color = ((i % 2)==0) ? "white" : "#D3D3D3";
            if ((mCurrentStatictics.mTMsg[i] > 0) && (mCurrentStatictics.mTErr[i] == 0))
                color = "#90EE90";
            html += QString("<td align=\"center\" bgcolor=\"%2\"> %1 </td>")
                    .arg(QString::number(mCurrentStatictics.mTMsg[i]))
                    .arg(color);
        }
        html += QString("</tr>");
        html += QString("<tr>");
        html += QString("<td align=\"left\"> %1 </td>").arg("ОШИБКИ:");
        for (int i=0; i<cnt; i++) {
            QString color;
            color = ((i % 2)==0) ? "white" : "#D3D3D3";
            if (mCurrentStatictics.mTErr[i] > 0)
                color = "#CD5C5C";
            html += QString("<td align=\"center\" bgcolor=\"%2\"> %1 </td>")
                    .arg(QString::number(mCurrentStatictics.mTErr[i])).arg(color);
        }
        html += QString("</tr>");
        html += QString("</table>");
        return html;
    };

    auto receiveTable = [this](void)->QString {
        QString html;
        int cnt = mCurrentStatictics.mRMsg.count();
        html += QString("<table align=\"center\" width=\"100%\" border=\"1\" cellpadding=\"0\" cellspacing=\"0\" >");
        html += QString("<tr>");
        html += QString("<td align=\"left\"> %1 </td>").arg("ПРИЁМНИКИ:");
        for (int i=0; i<cnt; i++) {
            html += QString("<td align=\"center\"> ПРМ%1 </td>").arg(QString::number(i+1));
        }
        html += QString("</tr>");
        html += QString("<tr>");
        html += QString("<td align=\"left\"> %1 </td>").arg("СООБЩЕНИЯ:");
        for (int i=0; i<cnt; i++) {
            QString color;
            color = ((i % 2)==0) ? "white" : "#D3D3D3";
            if ((mCurrentStatictics.mRMsg[i] > 0) && (mCurrentStatictics.mRErr[i] == 0))
                color = "#90EE90";
            html += QString("<td align=\"center\" bgcolor=\"%2\"> %1 </td>")
                    .arg(QString::number(mCurrentStatictics.mRMsg[i])).arg(color);
        }
        html += QString("</tr>");
        html += QString("<tr>");
        html += QString("<td align=\"left\"> %1 </td>").arg("ОШИБКИ:");
        for (int i=0; i<cnt; i++) {
            QString color;
            color = ((i % 2)==0) ? "white" : "#D3D3D3";
            if (mCurrentStatictics.mRErr[i] > 0)
                color = "#CD5C5C";
            html += QString("<td align=\"center\" bgcolor=\"%2\"> %1 </td>")
                    .arg(QString::number(mCurrentStatictics.mRErr[i])).arg(color);
        }
        html += QString("</tr>");
        html += QString("</table>");
        return html;
    };

    auto rkOutTable = [this](void)->QString {
        QString html;
        int cnt = mCurrentStatictics.mRkOut.count();
        html += QString("<table align=\"center\" width=\"100%\" border=\"1\" cellpadding=\"0\" cellspacing=\"0\" >");
        html += QString("<tr>");
        html += QString("<td align=\"left\"> %1 </td>").arg("РАЗОВЫЕ КОМАНДЫ:");
        for (int i=0; i<cnt; i++) {
            html += QString("<td align=\"center\"> ВЫХОД%1 </td>").arg(QString::number(i+1));
        }
        html += QString("</tr>");
        html += QString("<tr>");
        html += QString("<td align=\"left\"> %1 </td>").arg("КОЛИЧЕСТВО:");
        for (int i=0; i<cnt; i++) {
            QString color;
            color = ((i % 2)==0) ? "white" : "#D3D3D3";
            html += QString("<td align=\"center\" bgcolor=\"%2\"> %1 </td>")
                    .arg(QString::number(mCurrentStatictics.mRkOut[i])).arg(color);
        }
        html += QString("</tr>");
        html += QString("</table>");
        return html;
    };

    auto rkInTable = [this](void)->QString {
        QString html;
        int cnt = mCurrentStatictics.mRkIn.count();
        html += QString("<table align=\"center\" width=\"100%\" border=\"1\" cellpadding=\"0\" cellspacing=\"0\" >");
        html += QString("<tr>");
        html += QString("<td align=\"left\"> %1 </td>").arg("РАЗОВЫЕ КОМАНДЫ:");
        for (int i=0; i<cnt; i++) {
            html += QString("<td align=\"center\"> ВХОД%1 </td>").arg(QString::number(i+1));
        }
        html += QString("</tr>");
        html += QString("<tr>");
        html += QString("<td align=\"left\"> %1 </td>").arg("КОЛИЧЕСТВО:");
        for (int i=0; i<cnt; i++) {
            QString color;
            color = ((i % 2)==0) ? "white" : "#D3D3D3";
            html += QString("<td align=\"center\" bgcolor=\"%2\"> %1 </td>")
                    .arg(QString::number(mCurrentStatictics.mRkIn[i])).arg(color);
        }
        html += QString("</tr>");
        html += QString("</table>");
        return html;
    };

    QString html;
    html += QString("<font size=\"3\" face=\"Liberation Mono\">");
    html += QString("<p>");
    html += getDeviceName();
    html += QString("</p>");
    html += QString("<table align=\"center\" width=\"100%\" border=\"1\" cellpadding=\"0\" cellspacing=\"0\" >");
    html += QString("<tr>");
    html += QString("<td>");
    html += transmitTable();
    html += QString("</td>");
    html += QString("<td>");
    html += receiveTable();
    html += QString("</td>");
    html += QString("</tr>");
    html += QString("</table>");
    html += QString("<table align=\"center\" width=\"100%\" border=\"1\" cellpadding=\"0\" cellspacing=\"0\" >");
    html += QString("<tr>");
    html += QString("<td>");
    html += rkOutTable();
    html += QString("</td>");
    html += QString("<td>");
    html += rkInTable();
    html += QString("</td>");
    html += QString("</tr>");
    html += QString("</table>");
    html += QString("</font>");
    return html;
}

void WidgetDevice::setKey(int key)
{
    mKey = key;
}

void WidgetDevice::configure()
{
    setVisibleView();
    setUnusedUnVisibleView();
}

void WidgetDevice::setDeviceName(const QString &name)
{
    ui->labelDevName->setText(QString("ДЕВАЙС: %1")
                              .arg(name)
                              );
}

void WidgetDevice::setDeviceStatistics(const DeviceStatistics &devStat)
{
    setStatistics(devStat);
}


QString WidgetDevice::getDeviceName()
{
    return ui->labelDevName->text();
}


QStringList WidgetDevice::getStringStatistics()
{
    QStringList list;
    list.append(ui->labelDevName->text());
    QString trnmtMsg = QString("ПРД СБЩ: "), trnmtErr = QString("ПРД ОШБ: ");
    for (int index = 0; index < mCurrentStatictics.mTMsg.count(); index++) {
        trnmtMsg += QString("П%1=%2; ")
                .arg(QString::number(index+1))
                .arg(QString::number(mCurrentStatictics.mTMsg[index]));
        trnmtErr += QString("П%1=%2; ")
                .arg(QString::number(index+1))
                .arg(QString::number(mCurrentStatictics.mTErr[index]));
    }
    QString recvMsg = QString("ПРМ СБЩ: "), recvErr = QString("ПРМ ОШБ: ");
    for (int index = 0; index < mCurrentStatictics.mRMsg.count(); index++) {
        recvMsg += QString("П%1=%2; ")
                .arg(QString::number(index+1))
                .arg(QString::number(mCurrentStatictics.mRMsg[index]));
        recvErr += QString("П%1=%2; ")
                .arg(QString::number(index+1))
                .arg(QString::number(mCurrentStatictics.mRErr[index]));
    }
    QString rkOut = QString("РК ВЫХ: ");
    for (int index = 0; index < mCurrentStatictics.mRkOut.count(); index++) {
        rkOut += QString("РК%1=%2; ")
                .arg(QString::number(index+1))
                .arg(QString::number(mCurrentStatictics.mRkOut[index]));
    }
    QString rkIn = QString("РК ВХД: ");
    for (int index = 0; index < mCurrentStatictics.mRkIn.count(); index++) {
        rkIn += QString("РК%1=%2; ")
                .arg(QString::number(index+1))
                .arg(QString::number(mCurrentStatictics.mRkIn[index]));
    }
    list.append(trnmtMsg);
    list.append(trnmtErr);
    list.append(recvMsg);
    list.append(recvErr);
    list.append(rkOut);
    list.append(rkIn);
    return list;
}


void WidgetDevice::setDeviceId(int devid)
{
    mCurrentDevId = devid;
}

void WidgetDevice::setRevisionId(int revid)
{
    mCurrentRevId = revid;
}
