#ifndef WIDGETDEVICE_H
#define WIDGETDEVICE_H

#include <QFrame>
#include <QMap>
#include "interface/iwidgetdevice.h"

namespace Ui {
class WidgetDevice;
}

///
/// \brief The WidgetDevice class виджет для отображения наработанной статистики платы
///
class WidgetDevice : public QFrame, public IWidgetDevice
{
    Q_OBJECT

public:
    explicit WidgetDevice(QWidget *parent = nullptr);
    ~WidgetDevice();

    enum class BACKCLR { dark, light };
    ///
    /// \brief setBackClr управление фоном
    /// \param value тёный\светлый
    ///
    void setBackClr(BACKCLR value);
    ///
    /// \brief getHtmlText получить статистику в виде html таблицы
    /// \return html таблицы
    ///
    QString getHtmlText();
    ///
    /// \brief setKey установить идентифкатор девайса
    /// \param key
    ///
    void setKey(int key);

    void configure();

signals:
    ///
    /// \brief pressedKeyBlinked сигнал кнопки мигания
    /// \param key идентифкатор девайса
    ///
    void pressedKeyBlinked(int key);

private:
    ///
    /// \brief setVisibleView показать все элементы
    ///
    void setVisibleView();
    void setUnusedUnVisibleView();

    ///
    /// \brief setStatistics установка наработанной статистики для платы
    /// \param devStat наработанная статистика
    ///
    void setStatistics(const DeviceStatistics &devStat);
    ///
    /// \brief updateStatistics отобразить наработанную статистику для платы
    /// \param devStat наработанная статистика
    ///
    void updateStatistics(const DeviceStatistics &devStat);

private:
    Ui::WidgetDevice *ui;
    int mCurrentDevId = 0;///< текущий dev id
    int mCurrentRevId = 0;                  ///< текущая rev id
    BACKCLR mCurBackClr = BACKCLR::light;   ///< фон
    int mKey = -1;///< идентификатор девайса

    // IWidgetDevice interface
public:
    virtual void setDeviceName(const QString &name) override;
    virtual void setDeviceStatistics(const DeviceStatistics &devStat) override;
    virtual QString getDeviceName() override;
    virtual QStringList getStringStatistics() override;

    // IWidgetDevice interface
public:
    virtual void setDeviceId(int devid) override;
    virtual void setRevisionId(int revid) override;
};

#endif // WIDGETDEVICE_H
